import pytest

from ingredient_parser import PreProcessor


@pytest.fixture
def p():
    """Define an empty PreProcessor object to use for testing the PreProcessor
    class methods.
    """
    return PreProcessor("", defer_pos_tagging=True)


class TestPreProcessor__builtins__:
    def test__str__(self):
        """
        Test PreProcessor __str__
        """
        p = PreProcessor("1/2 cup chicken broth")
        truth = """Pre-processed recipe ingedient sentence
\t    Input: 1/2 cup chicken broth
\t  Cleaned: 0.5 cup chicken broth
\tTokenized: ['0.5', 'cup', 'chicken', 'broth']"""
        assert str(p) == truth

    def test__repr__(self):
        """
        Test PreProessor __repr__
        """
        p = PreProcessor("1/2 cup chicken broth")
        assert repr(p) == 'PreProcessor("1/2 cup chicken broth")'


class TestPreProcessor_replace_string_numbers:
    def test_spaces(self, p):
        """
        The string number, surrounded by spaces, is converted to a numeral
        """
        input_sentence = "Zest of one orange"
        assert p._replace_string_numbers(input_sentence) == "Zest of 1 orange"

    def test_start(self, p):
        """
        The string number, at the start of the sentence and followed by a space,
        is converted to a numeral
        """
        input_sentence = "Half of a lime"
        assert p._replace_string_numbers(input_sentence) == "0.5 of a lime"

    def test_parens(self, p):
        """
        The string number, at the beginning of a phrase inside parentheses,
        s converted to a numeral
        """
        input_sentence = "2 cups (three 12-ounce bags) frozen raspberries"
        assert (
            p._replace_string_numbers(input_sentence)
            == "2 cups (3 12-ounce bags) frozen raspberries"
        )

    def test_hyphen(self, p):
        """
        The string number, with a hyphen appended without a space,
        is converted to a numeral
        """
        input_sentence = "1 pound potatoes, peeled and cut into five-inch cubes"
        assert (
            p._replace_string_numbers(input_sentence)
            == "1 pound potatoes, peeled and cut into 5-inch cubes"
        )

    def test_substring(self, p):
        """
        The string number, appearing as a substring of another word,
        is not converted to a numeral
        """
        input_sentence = (
            "1 pound skinless, boneless monkfish fillets"  # "one" inside "boneless"
        )
        assert (
            p._replace_string_numbers(input_sentence)
            == "1 pound skinless, boneless monkfish fillets"
        )


class TestPreProcessor_replace_html_fractions:
    def test_half(self, p):
        """
        The HTML fraction &frac12; is converted to the unicode symbol ½
        There is no space between the preceding character and the start of the html
        fraction
        """
        input_sentence = "3&frac12; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3½ potatoes"

    def test_one_third(self, p):
        """
        The HTML fraction &frac13; is converted to the unicode symbol ⅓
        There is no space between the preceding character and the start of the html
        fraction
        """
        input_sentence = "3&frac13; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3⅓ potatoes"

    def test_two_thirds(self, p):
        """
        The HTML fraction &frac23; is converted to the unicode symbol ⅔
        There is no space between the preceding character and the start of the html
        fraction
        """
        input_sentence = "3&frac23; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3⅔ potatoes"

    def test_one_quarter(self, p):
        """
        The HTML fraction &frac14; is converted to the unicode symbol ¼
        There is no space between the preceding character and the start of the html
        fraction
        """
        input_sentence = "3&frac14; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3¼ potatoes"

    def test_three_quarters(self, p):
        """
        The HTML fraction &frac34; is converted to the unicode symbol ¾
        There is no space between the preceding character and the start of the html
        fraction
        """
        input_sentence = "3&frac34; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3¾ potatoes"

    def test_fifth(self, p):
        """
        The HTML fraction &frac15; is converted to the unicode symbol ⅕
        """
        input_sentence = "3 &frac15; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅕ potatoes"

    def test_two_fifth(self, p):
        """
        The HTML fraction &frac25; is converted to the unicode symbol ⅖
        """
        input_sentence = "3 &frac25; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅖ potatoes"

    def test_three_fifth(self, p):
        """
        The HTML fraction &frac35; is converted to the unicode symbol ⅗
        """
        input_sentence = "3 &frac35; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅗ potatoes"

    def test_four_fifth(self, p):
        """
        The HTML fraction &frac45; is converted to the unicode symbol ⅘
        """
        input_sentence = "3 &frac45; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅘ potatoes"

    def test_one_sixth(self, p):
        """
        The HTML fraction &frac16; is converted to the unicode symbol ⅙
        """
        input_sentence = "3 &frac16; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅙ potatoes"

    def test_five_sixths(self, p):
        """
        The HTML fraction &frac56; is converted to the unicode symbol ⅚
        """
        input_sentence = "3 &frac56; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅚ potatoes"

    def test_one_eighth(self, p):
        """
        The HTML fraction &frac18; is converted to the unicode symbol ⅛
        """
        input_sentence = "3 &frac18; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅛ potatoes"

    def test_three_eighths(self, p):
        """
        The HTML fraction &frac38; is converted to the unicode symbol ⅜
        """
        input_sentence = "3 &frac38; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅜ potatoes"

    def test_five_eighths(self, p):
        """
        The HTML fraction &frac58; is converted to the unicode symbol ⅝
        """
        input_sentence = "3 &frac58; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅝ potatoes"

    def test_seven_eighths(self, p):
        """
        The HTML fraction &frac78; is converted to the unicode symbol ⅞
        """
        input_sentence = "3 &frac78; potatoes"
        assert p._replace_html_fractions(input_sentence) == "3 ⅞ potatoes"


class TestPreProcessor_replace_unicode_fractions:
    def test_half(self, p):
        """
        The unicode fraction ½ is converted to 1/2
        There is no space between the preceding character and the unicode fraction
        """
        input_sentence = "3½ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3 1/2 potatoes"

    def test_third(self, p):
        """
        The unicode fraction ⅓ is converted to 1/3
        There is no space between the preceding character and the unicode fraction
        """
        input_sentence = "3⅓ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3 1/3 potatoes"

    def test_two_thirds(self, p):
        """
        The unicode fraction ⅔ is converted to 2/3
        There is no space between the preceding character and the unicode fraction
        """
        input_sentence = "3⅔ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3 2/3 potatoes"

    def test_quarter(self, p):
        """
        The unicode fraction ¼ is converted to 1/4
        There is no space between the preceding character and the unicode fraction
        """
        input_sentence = "3¼ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3 1/4 potatoes"

    def test_three_quarters(self, p):
        """
        The unicode fraction ¾ is converted to 3/4
        There is no space between the preceding character and the unicode fraction
        """
        input_sentence = "3¾ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3 3/4 potatoes"

    def test_fifth(self, p):
        """
        The unicode fraction ⅕ is converted to 1/5
        """
        input_sentence = "3 ⅕ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  1/5 potatoes"

    def test_two_fifth(self, p):
        """
        The unicode fraction ⅖ is converted to 2/5
        """
        input_sentence = "3 ⅖ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  2/5 potatoes"

    def test_three_fifth(self, p):
        """
        The unicode fraction ⅗ is converted to 3/5
        """
        input_sentence = "3 ⅗ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  3/5 potatoes"

    def test_four_fifth(self, p):
        """
        The unicode fraction ⅘ is converted to 4/5
        """
        input_sentence = "3 ⅘ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  4/5 potatoes"

    def test_one_sixth(self, p):
        """
        The unicode fraction ⅙ is converted to 1/6
        """
        input_sentence = "3 ⅙ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  1/6 potatoes"

    def test_five_sixths(self, p):
        """
        The unicode fraction ⅚ is converted to 5/6
        """
        input_sentence = "3 ⅚ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  5/6 potatoes"

    def test_one_eighth(self, p):
        """
        The unicode fraction ⅛ is converted to 1/8
        """
        input_sentence = "3 ⅛ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  1/8 potatoes"

    def test_three_eighths(self, p):
        """
        The unicode fraction ⅜ is converted to 3/8
        """
        input_sentence = "3 ⅜ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  3/8 potatoes"

    def test_five_eighths(self, p):
        """
        The unicode fraction ⅝ is converted to 5/8
        """
        input_sentence = "3 ⅝ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  5/8 potatoes"

    def test_seven_eighths(self, p):
        """
        The unicode fraction ⅞ is converted to 7/8
        """
        input_sentence = "3 ⅞ potatoes"
        assert p._replace_unicode_fractions(input_sentence) == "3  7/8 potatoes"


class TestPreProcessor_replace_fake_fractions:
    def test_less_than_one(self, p):
        """
        The fake fraction 1/2 is replaced with 0.5
        """
        input_sentence = "1/2 cup sugar"
        assert p._replace_fake_fractions(input_sentence) == "0.5 cup sugar"

    def test_greater_than_one(self, p):
        """
        The fake fraction 3 1/3 is replaced with 3.333
        """
        input_sentence = "1 pound melted butter, about 3 1/3 cups"
        assert (
            p._replace_fake_fractions(input_sentence)
            == "1 pound melted butter, about 3.333 cups"
        )

    def test_no_fraction(self, p):
        """
        There is no fake fraction in the input
        """
        input_sentence = "pinch of salt"
        assert p._replace_fake_fractions(input_sentence) == input_sentence

    def test_leading_space(self, p):
        """
        The fake fraction 1/2 is replaced with 0.5
        The input sentence starts with a space
        """
        input_sentence = " 1/2 cup sugar"
        assert p._replace_fake_fractions(input_sentence) == " 0.5 cup sugar"


class TestPreProcessor_split_quantity_and_units:
    def test_basic(self, p):
        """
        A space is inserted between the integer quantity and the unit
        """
        input_sentence = "100g plain flour"
        assert p._split_quantity_and_units(input_sentence) == "100 g plain flour"

    def test_decimal(self, p):
        """
        A space is inserted between the decimal quantity and the unit
        """
        input_sentence = "2.5cups orange juice"
        assert p._split_quantity_and_units(input_sentence) == "2.5 cups orange juice"

    def test_inch(self, p):
        """
        No space is inserted between the quantity and the inches symbol
        """
        input_sentence = '2.5" square chocolate'
        assert p._split_quantity_and_units(input_sentence) == '2.5" square chocolate'


class TestPreProcessor_singlarise_units:
    def test_embedded(self, p):
        """
        The unit "cups" is replaced with "cup"
        """
        input_sentence = ["2.5", "cups", "beer"]
        tokenised_sentence, indices = p._singlarise_units(input_sentence)
        assert tokenised_sentence == ["2.5", "cup", "beer"]
        assert indices == [1]

    def test_capitalised(self, p):
        """
        The unit "Boxes" is replaced with "Box", with the capitalisation maintained
        """
        input_sentence = ["2.5", "Boxes", "Candy"]
        tokenised_sentence, indices = p._singlarise_units(input_sentence)
        assert tokenised_sentence == ["2.5", "Box", "Candy"]
        assert indices == [1]

    def test_start(self, p):
        """
        The unit "leaves" is replaced with "leaf"
        """
        input_sentence = ["leaves", "of", "basil"]
        tokenised_sentence, indices = p._singlarise_units(input_sentence)
        assert tokenised_sentence == ["leaf", "of", "basil"]
        assert indices == [0]

    def test_start_capitalised(self, p):
        """
        The unit "wedges" is replaced with "wedge", with the capitalisation maintained
        """
        input_sentence = ["Wedges", "of", "lemon"]
        tokenised_sentence, indices = p._singlarise_units(input_sentence)
        assert tokenised_sentence == ["Wedge", "of", "lemon"]
        assert indices == [0]

    def test_multiple_units(self, p):
        """
        The units "tablespoons" and "teaspoons" are replaced with "tablespoon" and
        "teaspoon" respectively
        """
        input_sentence = ["2", "tablespoons", "plus", "2", "teaspoons"]
        tokenised_sentence, indices = p._singlarise_units(input_sentence)
        assert tokenised_sentence == ["2", "tablespoon", "plus", "2", "teaspoon"]
        assert indices == [1, 4]


class TestPreProcessor_is_unit:
    def test_true(self, p):
        """
        "glass" is a unit
        """
        assert p._is_unit("glass")

    def test_false(self, p):
        """
        "watt" is not a unit
        """
        assert not p._is_unit("watt")


class TestPreProcessor_is_numeric:
    def test_integer(self, p):
        """
        "1" is numeric
        """
        assert p._is_numeric("1")

    def test_decimal(self, p):
        """
        "2.667" is numeric
        """
        assert p._is_numeric("2.667")

    def test_integer_range(self, p):
        """
        "1-2" is numeric
        """
        assert p._is_numeric("1-2")

    def test_decimal_range(self, p):
        """
        "3.5-5.5" is numeric
        """
        assert p._is_numeric("3.5-5.5")

    def test_mixed_range(self, p):
        """
        "1-1.5" is numeric
        """
        assert p._is_numeric("1-1.5")

    def test_false(self, p):
        """
        "1/2" is not numeric
        """
        assert not p._is_numeric("1/2")

    def test_false_range(self, p):
        """
        "red-wine" is not numeric
        """
        assert not p._is_numeric("red-wine")


class TestPreProcessor_is_capitalised:
    def test_capitalised(self, p):
        """
        "Cheese" is capitalised
        """
        assert p._is_capitalised("Cheese")

    def test_embeded_capital(self, p):
        """
        "lemon-Zest" is not capitalised
        """
        assert not p._is_capitalised("lemon-Zest")

    def test_no_captials(self, p):
        """
        "sausage" is not capitalised
        """
        assert not p._is_capitalised("sausage")


class TestPreProcessor_is_inside_parentheses:
    def test_inside(self):
        """
        Token index is inside parens
        """
        input_sentence = "8-10 teaspoons pine nuts (ground), toasted"
        p = PreProcessor(input_sentence)
        assert p._is_inside_parentheses(5)

    def test_before(self):
        """
        Token index is before parens
        """
        input_sentence = "8-10 teaspoons pine nuts (ground), toasted"
        p = PreProcessor(input_sentence)
        assert not p._is_inside_parentheses(2)

    def test_after(self):
        """
        Token index is before parens
        """
        input_sentence = "8-10 teaspoons pine nuts (ground), toasted"
        p = PreProcessor(input_sentence)
        assert not p._is_inside_parentheses(7)

    def test_open_parens(self):
        """
        Token index is (
        """
        input_sentence = "8-10 teaspoons pine nuts (ground), toasted"
        p = PreProcessor(input_sentence)
        assert p._is_inside_parentheses(4)

    def test_close_parens(self):
        """
        Token index is (
        """
        input_sentence = "8-10 teaspoons pine nuts (ground), toasted"
        p = PreProcessor(input_sentence)
        assert p._is_inside_parentheses(6)


class TestPreProcess_follows_plus:
    def test_no_plus(self):
        """
        No "plus" in input
        """
        input_sentence = "freshly ground black pepper"
        p = PreProcessor(input_sentence)
        assert not p._follows_plus(2)

    def test_before_plus(self):
        """
        Token index is before "plus"
        """
        input_sentence = "freshly ground black pepper, plus more to taste"
        p = PreProcessor(input_sentence)
        assert not p._follows_plus(1)

    def test_after_plus(self):
        """
        Token index is after "plus"
        """
        input_sentence = "freshly ground black pepper, plus more to taste"
        p = PreProcessor(input_sentence)
        assert p._follows_plus(7)

    def test_index_is_plus(self):
        """
        Token at index is "plus"
        """
        input_sentence = "freshly ground black pepper, plus more to taste"
        p = PreProcessor(input_sentence)
        assert not p._follows_plus(5)

    def test_index_is_plus_and_follows_plus(self):
        """
        Token at index is "plus" and follows another "plus"
        """
        input_sentence = (
            "freshly ground black pepper, plus white pepper, plus more to taste"
        )
        p = PreProcessor(input_sentence)
        assert p._follows_plus(9)


class TestPreProcess_follows_comma:
    def test_no_comma(self):
        """
        No comma in input
        """
        input_sentence = "freshly ground black pepper"
        p = PreProcessor(input_sentence)
        assert not p._follows_comma(2)

    def test_before_comma(self):
        """
        Token index is before comma
        """
        input_sentence = "freshly ground black pepper, to taste"
        p = PreProcessor(input_sentence)
        assert not p._follows_comma(1)

    def test_after_comma(self):
        """
        Token index is after comma
        """
        input_sentence = "freshly ground black pepper, to taste"
        p = PreProcessor(input_sentence)
        assert p._follows_comma(5)

    def test_index_is_comma(self):
        """
        Token at index is comma
        """
        input_sentence = "freshly ground black pepper, to taste"
        p = PreProcessor(input_sentence)
        assert not p._follows_comma(4)

    def test_index_is_comma_and_follows_comma(self):
        """
        Token at index is comma and follows another comma
        """
        input_sentence = "freshly ground black pepper, or white pepper, to taste"
        p = PreProcessor(input_sentence)
        assert p._follows_comma(8)


class TestPreProcessor__is_stop_word:
    def test_common(self, p):
        """
        Validate common stop words
        """
        for word in ["of", "in", "the", "or", "and", "a", "an", "at", "by"]:
            assert p._is_stop_word(word)

    def test_false(self, p):
        """
        Test non-stop words
        """
        for word in ["onion", "kg", "pound", "rice", "taste"]:
            assert not p._is_stop_word(word)



import pytest

from ingredient_parser.utils import (
    average,
    find_idx,
    fix_punctuation,
    group_consecutive_idx,
    join_adjacent,
    pluralise_units,
)


class TestUtils_find_idx:
    def test_name(self):
        """
        Returns indices of NAME labels
        """
        input_labels = ["QTY", "UNIT", "NAME", "NAME", "COMMENT", "COMMENT"]
        assert find_idx(input_labels, "NAME") == [2, 3]

    def test_comment(self):
        """
        Returns indices of COMMENT labels
        """
        input_labels = ["QTY", "UNIT", "NAME", "NAME", "COMMENT", "COMMENT"]
        assert find_idx(input_labels, "COMMENT") == [4, 5]

    def test_qty(self):
        """
        Returns indices of QTY labels
        """
        input_labels = ["QTY", "UNIT", "NAME", "NAME", "COMMENT", "COMMENT"]
        assert find_idx(input_labels, "QTY") == [0]

    def test_unit(self):
        """
        Returns indices of UNIT labels
        """
        input_labels = ["QTY", "UNIT", "NAME", "NAME", "COMMENT", "COMMENT"]
        assert find_idx(input_labels, "UNIT") == [1]

    def test_other(self):
        """
        Returns indices of OTHER labels
        """
        input_labels = [
            "QTY",
            "UNIT",
            "NAME",
            "NAME",
            "OTHER",
            "COMMENT",
            "COMMENT",
            "OTHER",
        ]
        assert find_idx(input_labels, "OTHER") == [4, 7]

    def test_comment_including_comma(self):
        """
        Returns indices of COMMENT labels, including COMMA
        """
        input_labels = ["QTY", "UNIT", "NAME", "NAME", "COMMA", "COMMENT", "COMMENT"]
        assert find_idx(input_labels, "COMMENT") == [4, 5, 6]


class TestUtils_group_consecutive_indices:
    def test_single_group(self):
        """
        Return single group
        """
        input_indices = [0, 1, 2, 3, 4]
        groups = group_consecutive_idx(input_indices)
        assert [list(g) for g in groups] == [input_indices]

    def test_multiple_groups(self):
        """
        Return groups of consecutive indices
        """
        input_indices = [0, 1, 2, 4, 5, 6, 8, 9]
        groups = group_consecutive_idx(input_indices)
        assert [list(g) for g in groups] == [[0, 1, 2], [4, 5, 6], [8, 9]]


class TestUtils_join_adjacent:
    def test_single_group(self):
        """
        Join all input strings into single string
        """
        input_strings = ["a", "b", "c", "d", "e", "f"]
        input_indices = [1, 2, 3, 4]
        assert join_adjacent(input_strings, input_indices) == "b c d e"

    def test_multiple_groups(self):
        """
        Join input strings into two string
        """
        input_strings = ["a", "b", "c", "d", "e", "f"]
        input_indices = [0, 1, 3, 4, 5]
        assert join_adjacent(input_strings, input_indices) == ["a b", "d e f"]

    def test_no_groups(self):
        """
        Return empty string
        """
        input_strings = ["a", "b", "c", "d", "e", "f"]
        input_indices = []
        assert join_adjacent(input_strings, input_indices) == ""


class TestUtils_fix_punctuation:
    def test_opening_paren(self):
        """
        Space following opening parenthesis is removed
        """
        input_sentence = "2 cups ( high quality) bread flour"
        assert fix_punctuation(input_sentence) == "2 cups (high quality) bread flour"

    def test_closing_paren(self):
        """
        Space preceding closing parenthesis is removed
        """
        input_sentence = "2 cups (high quality ) bread flour"
        assert fix_punctuation(input_sentence) == "2 cups (high quality) bread flour"

    def test_comma(self):
        """
        Space preceding comma is removed
        """
        input_sentence = "salt and pepper , to taste"
        assert fix_punctuation(input_sentence) == "salt and pepper, to taste"

    def test_all(self):
        """
        Spaces following opening parenthesis, preceding closing parenthesis,
        preceding comma are removed
        """
        input_sentence = "2 teaspoons ( kosher ) salt , to taste"
        assert fix_punctuation(input_sentence) == "2 teaspoons (kosher) salt, to taste"

    def test_starting_comma(self):
        """
        Leading comma in sentence removed
        """
        input_sentence = ", or to taste"
        assert fix_punctuation(input_sentence) == "or to taste"


class TestUtils_pluralise_units:
    def test_single(self):
        """
        Each singular unit gets pluralised
        """
        assert pluralise_units("teaspoon") == "teaspoons"
        assert pluralise_units("cup") == "cups"
        assert pluralise_units("loaf") == "loaves"
        assert pluralise_units("leaf") == "leaves"
        assert pluralise_units("chunk") == "chunks"
        assert pluralise_units("Box") == "Boxes"
        assert pluralise_units("Wedge") == "Wedges"

    def test_embedded(self):
        """
        The unit embedded in each sentence gets pluralised
        """
        assert pluralise_units("2 tablespoon olive oil") == "2 tablespoons olive oil"
        assert (
            pluralise_units("3 cup (750 milliliter) milk")
            == "3 cups (750 milliliters) milk"
        )


@pytest.fixture
def labels():
    """
    Return labels for testing average function
    """
    return [
        "QTY",
        "UNIT",
        "NAME",
        "NAME",
        "COMMA",
        "COMMENT",
        "COMMENT",
        "COMMENT",
        "COMMENT",
    ]


@pytest.fixture
def scores():
    """
    Return scores for each labels for testing average function
    """
    return [
        0.9995912554903719,
        0.9973499215679792,
        0.9374103977762065,
        0.9144715768558962,
        0.9997843568900641,
        0.9999263435592202,
        0.9995634086142597,
        0.9999196125676765,
        0.9959501880520711,
    ]


class TestUtils_average:
    def test_average_qty(self, labels, scores):
        """
        Average QTY label score
        """
        assert average(labels, scores, "QTY") == 0.9996

    def test_average_unit(self, labels, scores):
        """
        Average UNIT label score
        """
        assert average(labels, scores, "UNIT") == 0.9973

    def test_average_name(self, labels, scores):
        """
        Average NAME label score
        """
        assert average(labels, scores, "NAME") == 0.9259

    def test_average_comment(self, labels, scores):
        """
        Average COMMENT label score
        """
        assert average(labels, scores, "COMMENT") == 0.9988

    def test_average_other(self, labels, scores):
        """
        Average OTHER label score
        """
        assert average(labels, scores, "OTHER") == 0



from .parsers import parse_ingredient, parse_multiple_ingredients
from .preprocess import PreProcessor
from .regex_parser import parse_ingredient_regex

__all__ = [
    "parse_ingredient",
    "parse_multiple_ingredients",
    "parse_ingredient_regex",
    "PreProcessor",
]

__version__ = "0.1.0-beta2"



#!/usr/bin/env python3

import re
from itertools import groupby
from operator import itemgetter
from typing import Generator, Iterator

from ._constants import UNITS


def find_idx(labels: list[str], key: str) -> list[int]:
    """Find indices of elements in list matching key.

    Return the indices of every element in the input list whose value is equal to the
    given key.
    If there is an element with the COMMA label before an element with label key,
    include this in the list of matched indices

    Parameters
    ----------
    labels : list[str]
        List to search for key in
    key : str
        Key to find in list

    Returns
    -------
    list[int]
        List of indices of elements with label key
        Includes elements with label COMMA if the element is previous to a key label

    Examples
    -------
    >>> find_idx(["QTY", "UNIT", "NAME", "NAME", "COMMENT", "COMMENT"], "NAME")
    [2, 3]

    >>> find_idx(["QTY", "UNIT", "NAME", "COMMA", "COMMENT", "COMMENT"], "COMMENT")
    [4, 5, 6]
    """
    matches = []
    prev_el = ""
    for idx, el in enumerate(labels):
        if el == key:
            if prev_el == "COMMA":
                matches.append(idx - 1)
            matches.append(idx)

        prev_el = el
    return matches


def group_consecutive_idx(idx: list[int]) -> Generator[Iterator[int], None, None]:
    """Yield groups of consecutive indices

    Given a list of integers, yield groups of integers where the value of each in a
    group is adjacent to the previous element's value.

    Parameters
    ----------
    idx : list[int]
        List of indices

    Yields
    ------
    list[list[int]]
        List of lists, where each sub-list contains consecutive indices

    Examples
    -------
    >>> groups = group_consecutive_idx([0, 1, 2, 4, 5, 6, 8, 9])
    >>> [list(g) for g in groups]
    [[0, 1, 2], [4, 5, 6], [8, 9]]
    """
    for k, g in groupby(enumerate(idx), key=lambda x: x[0] - x[1]):
        yield map(itemgetter(1), g)


def join_adjacent(tokens: list[str], idx: list[int]) -> str | list[str]:
    """Join tokens with adjacent indices in idx list into strings.

    Given a list of tokens and list of indices for token with a particular value,
    join all the token with adjacent indices in the idx list into space seperated
    strings.

    If idx is an empty list, return an empty string.

    If there is only one group of adjacent values in idx, return a string.

    If there are multiple groups of adjacent values in idx, return a list of strings.

    Parameters
    ----------
    tokens : list[str]
        List of ingredient sentence tokens
    idx : list[int]
        Indices of tokens to group and join

    Returns
    -------
    str | list[str]
        List of strings, with adjacent tokens joined
        If the list only contains one element, return as a string

    Examples
    --------
    >>> join_adjacent(["a", "b", "c", "d", "e", "f"], [0, 1, 3, 4, 5])
    ['a b', 'd e f']
    """
    grouped = []
    for group in group_consecutive_idx(idx):
        joined = " ".join([tokens[idx] for idx in group])
        grouped.append(joined)

    if len(grouped) == 0:
        return ""
    elif len(grouped) == 1:
        return grouped[0]
    else:
        return grouped


def average(labels: list[str], scores: list[float], key: str) -> float:
    """Average the scores for labels matching key

    Given a particular label (key), find the indices of the labels that have that key.
    Then use those indices to take the average of the associated score in the scores
    input.

    Parameters
    ----------
    labels : list[str]
        List of labels to search key for
    scores : list[float]
        Confidence score for each label
    key : str
        Key to calculate confidence for

    Returns
    -------
    float
        Confidence, average of all labels with given key
    """
    score_list = []
    for idx, el in enumerate(labels):
        if el == key:
            score_list.append(scores[idx])

    if len(score_list) == 0:
        return 0

    average = sum(score_list) / len(score_list)
    return round(average, 4)


def fix_punctuation(sentence: str) -> str:
    """Fix punctuation when joining a list into a string

    1. Remove the ", " from start of the sentence
    2. Remove the space following an opening parens "(" and the space preceeding a
       closing parens ")" caused by using " ".join to turn a list into a sentence
    3. Remove the space preceeding a comma

    Parameters
    ----------
    sentence : str
        Sentence in which to fix punctuation

    Returns
    -------
    str
        Modified sentence

    Examples
    -------
    >>> fix_punctuation(", some words")
    'some words'

    >>> fix_punctuation("( text in brackets )")
    '(text in brackets)'

    >>> fix_punctuation("a comma follows this text ,")
    'a comma follows this text,'
    """
    # Let's not have any sentence fragments start with a comma
    if sentence.startswith(", "):
        sentence = sentence[2:]

    return sentence.replace("( ", "(").replace(" )", ")").replace(" ,", ",")


def pluralise_units(sentence: str) -> str:
    """Pluralise units in the sentence.

    Use the same UNITS dictionary as PreProcessor to make any units in sentence plural

    Parameters
    ----------
    sentence : str
        Input sentence

    Returns
    -------
    str
        Input sentence with any words in the values of UNITS replaced with their plural
        version

    Examples
    --------
    >>> pluralise_units("2 bag")
    '2 bags'

    >>> pluralise_units("13 ounce")
    '13 ounces'

    >>> pluralise_units("1.5 loaf bread")
    '1.5 loaves bread'
    """
    for plural, singular in UNITS.items():
        sentence = re.sub(rf"\b({singular})\b", f"{plural}", sentence)

    return sentence
